<?php

namespace Drupal\sms_zenziva\Plugin\SmsGateway;

use GuzzleHttp\Exception\GuzzleException;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelTrait;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\sms\Message\SmsDeliveryReport;
use Drupal\sms\Message\SmsMessageReportStatus;
use Drupal\sms\Message\SmsMessageResultStatus;
use GuzzleHttp\Client;
use Drupal\Core\Form\FormStateInterface;
use Drupal\sms\Message\SmsMessageInterface;
use Drupal\sms\Message\SmsMessageResult;
use Drupal\sms\Plugin\SmsGatewayPluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * @SmsGateway(
 *   id = "zenziva",
 *   label = @Translation("Reguler Zenziva"),
 *   outgoing_message_max_recipients = 1,
 *   reports_push = TRUE,
 *   incoming = TRUE,
 *   incoming_route = TRUE
 * )
 */
class RegulerZenziva extends SmsGatewayPluginBase implements ContainerFactoryPluginInterface {

  use LoggerChannelTrait;

  private const STATUS_MAPPING = [
    0 => SmsMessageReportStatus::DELIVERED,
    1 => SmsMessageReportStatus::INVALID_RECIPIENT,
    5 => SmsMessageResultStatus::ACCOUNT_ERROR,
    6 => SmsMessageReportStatus::CONTENT_INVALID,
    89 => SmsMessageResultStatus::EXCESSIVE_REQUESTS,
    99 => SmsMessageResultStatus::NO_CREDIT,
  ];

  /**
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, Client $client, LoggerChannelFactoryInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->httpClient = $client;
    $this->setLoggerFactory($logger);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): RegulerZenziva {
    return new static($configuration, $plugin_id, $plugin_definition, $container->get('http_client'), $container->get('logger.factory'));
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration(): array {
    return [
      'userkey' => '',
      'passkey' => '',
      'url' => 'https://reguler.zenziva.net/apps/smsapi.php',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildConfigurationForm($form, $form_state);

    $config = $this->getConfiguration();

    $form['zenziva'] = [
      '#type' => 'details',
      '#title' => $this->t('Zenziva'),
      '#open' => TRUE,
    ];

    $form['zenziva']['help'] = [
      '#type' => 'html_tag',
      '#tag' => 'p',
      '#value' => $this->t('API keys can be found at <a href="@url">@url</a>.', [
        '@url' => 'https://reguler.zenziva.net/apps/api.php',
      ]),
    ];

    $form['zenziva']['url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL'),
      '#default_value' => $config['url'],
      '#required' => TRUE,
    ];

    $form['zenziva']['userkey'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Userkey'),
      '#default_value' => $config['userkey'],
      '#placeholder' => 'XXXXXX',
      '#required' => TRUE,
    ];

    $form['zenziva']['passkey'] = [
      '#type' => 'password',
      '#title' => $this->t('Passkey'),
      '#default_value' => $config['passkey'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['userkey'] = trim($form_state->getValue('userkey'));
    $this->configuration['passkey'] = trim($form_state->getValue('passkey'));
    $this->configuration['url'] = trim($form_state->getValue('url'));
  }

  /**
   * {@inheritdoc}
   */
  public function send(SmsMessageInterface $sms_message): SmsMessageResult {
    $recipient = $sms_message->getRecipients()[0];
    $result = new SmsMessageResult();

    $config = $this->configuration;

    $message = $sms_message->getMessage();

    $report = new SmsDeliveryReport();
    $report->setRecipient($recipient);
    $report->setStatus(SmsMessageReportStatus::QUEUED);
    $report->setMessageId($config['url']);

    try {

      $response = $this->httpClient->request('GET', $config['url'], [
        'query' => [
          'userkey' => $config['userkey'],
          'passkey' => $config['passkey'],
          'nohp' => $recipient,
          'pesan' => $message,
        ],
      ]);

      $response_content = new \SimpleXMLElement($response->getBody()->getContents());

      $response_status = (int) $response_content->message[0]->status->__toString();

      $status = SmsMessageReportStatus::ERROR;
      $status_message = SmsMessageReportStatus::ERROR;

      if ($response_status === 0) {
        $status = SmsMessageReportStatus::DELIVERED;
      }
      if (in_array($response_status, self::STATUS_MAPPING)) {
        $status_message = self::STATUS_MAPPING[$status];
      }

      $report->setStatus($status);
      $report->setStatusMessage($status_message);

      $report->getStatus() !== SmsMessageReportStatus::ERROR ?: $this->getLogger('sms_zenziva')
        ->error($this->t('There is a problem when sending message: @message to @number, because of @error.', [
          '@message' => $message,
          '@number' => $report->getRecipient(),
          '@error' => $report->getStatusMessage(),
        ]));

    }
    catch (GuzzleException $exception) {
      $report->setStatus(SmsMessageReportStatus::ERROR);
      $report->setStatusMessage(SmsMessageResultStatus::ERROR);

      $this->getLogger('sms_zenziva')
        ->error($this->t('There is a problem when sending message: @message to @number, because of @error.', [
          '@message' => $message,
          '@number' => $report->getRecipient(),
          '@error' => $report->getStatusMessage(),
        ]));

    }

    $result->addReport($report);

    return $result;
  }

}
